#-------------------------------------------------------------------------------
# Name:        File Download to Host Interface Demo
# Purpose:     To illistrait how to use the TCP/UDP cLient IWIN AT commands to
#              download a file from a HTTP server and send the data to the host
#              processor. This demo is useful for OEM's that want to:
#                   1. Downlaod a file using eS-Wifi AT commands
#                   2. Setup UART , connect to a netowrk and downloaded a file
#                   2. Download a file to update your Host micro
#                   3. Downlaod a file and use the "ZU" command to update the
#                      eS-Wifi firwmare. Using the ZU commands requires that you
#                      have a ST bootolader or equivalent protocol on your host
#                      to update the firmware on the eS-WIFi module.
# Author:      sep
#
# Created:     03/18/2015
# Copyright:   (c) Inventek Systems, LLC. 2015
#
# Name:        INVENTEK WIRELESS INTEROPERABILITY NETWORK (IWIN), AT Command Software
# Purpose:     To provide Inventek customers Python scripts illustrating how to
#              use the IWIN AT Command Software.
#
#------------------------------------------------------------------------------
#LICENSE:
#   THE IWIN PROGRAM IS FREE SOFTWARE FOR INVENTEK CUSTOMERS ONLY.
#   INVENTEK SYSTEMS RETAINS ALL OWNERSHIP AND INTELLECTUAL PROPERTY RIGHTS
#   IN THE IWIN CODE ACCOMPANYING THIS MESSAGE AND IN ALL DERIVATIVES HERETO.
#   USER MAY USE THE IWIN CODE, AND ANY DERIVATIVES CREATED BY ANY PERSON OR
#   ENTITY BY OR ON USER?S BEHALF, EXCLUSIVELY WITH INVENTEK'S PROPRIETARY
#   PRODUCTS.
#
#   USER AGREES THAT USER IS SOLELY RESPONSIBLE FOR TESTING THE CODE AND
#   DETERMINING IT?S SUITABILITY.  INVENTEK HAS NO OBLIGATION TO MODIFY, TEST,
#   CERTIFY, OR SUPPORT THE CODE.
#
#   USER ACCEPTANCE AND/OR USE OF THE IWIN CODE CONSTITUTES AGREEMENT TO THE
#   TERMS AND CONDITIONS OF THIS NOTICE. IWIN CAN BE REDISTRIBUTED AND/OR
#   MODIFIED BY INVENTEK CUSTOMERS ONLY AND UNDER THE TERMS OF THE GNU GENERAL
#   PUBLIC LICENSE AS PUBLISHED BY THE FREE SOFTWARE FOUNDATION, EITHER VERSION
#   3 OF THE LICENSE, OR (AT USER?S OPTION) ANY LATER VERSION.
#
#   THIS PROGRAM IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL, BUT WITHOUT
#   ANY WARRANTY; WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS
#   FOR A PARTICULAR PURPOSE.  REFERENCE THE GNU GENERAL PUBLIC LICENSE FOR MORE
#   DETAILS.
#
#   CODE ACCOMPANYING THIS MESSAGE IS SUPPLIED BY INVENTEK "AS IS".  NO WARRANTIES,
#   WHETHER EXPRESSED, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO,
#   IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS FOR A
#   PARTICULAR PURPOSE APPLY TO THIS CODE, IT?S INTERACTION WITH INVENTEK 'S
#   PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.
#
#   USER ACKNOWLEDGES AND AGREES THAT, IN NO EVENT, SHALL INVENTEK BE LIABLE,
#   WHETHER IN CONTRACT, WARRANTY, TORT (INCLUDING NEGLIGENCE OR BREACH OF
#   STATUTORY DUTY), STRICT LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE,
#   FOR ANY INDIRECT, SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL
#   LOSS, DAMAGE, FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE CODE,
#   HOWSOEVER CAUSED, EVEN IF INVENTEK HAS BEEN ADVISED OF THE POSSIBILITY OR THE
#   DAMAGES ARE FORESEEABLE.  TO THE FULLEST EXTENT ALLOWABLE BY LAW, INVENTEK 'S
#   TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS CODE, SHALL NOT
#   EXCEED THE PRICE PAID DIRECTLY TO INVENTEK SPECIFICALLY TO HAVE THIS CODE
#   DEVELOPED.
#
#   USER SHOULD HAVE RECEIVED A COPY OF THE GNU GENERAL PUBLIC LICENSE ALONG
#   WITH THIS PROGRAM. IF NOT, SEE <HTTP://WWW.GNU.ORG/LICENSES/>.
#-------------------------------------------------------------------------------
#
# Version:      0.0         sep         20150313    Intial release
#               0.1         sep         20150916    Fixed bug with data being
#                                                   attched to HTTP response
#               0.2         sep         20150917    Minor update
#               0.3         sep         20180208    Updated for user input,
#                                                   to use R0 or R4 command,
#                                                   now used file name from URL
#                                                   for out file.
#               0.4         sep         20190220    Updated to support https
#
# Usage:        1.  Start Server that will be connected. Recommend: http://www.rejetto.com/hfs/ (http Only)
#               2.  Connect eS-Wifi module to UART on PC
#               3.  Optional: Setup your defualt AP connection by entering the following:
#                   SSID = "XXXXXXXX"       #SSID of AP
#                   PSWD = "XXXXXXXX"       #Password of AP
#                   SEC="=4"                #Security type of AP:
#                                           0-Open, 1-WEP, 2-WPA,
#                                           3-WPA2-AES, 4-WPA2-Mixed
#              4.  Run the script.
#
# The python script will:
#                        * Setup the UART port
#                        * Prompt the user for SSID, PASSWORD, SECURITY, and URL
#                        * Connect to your network
#                        * Do a DNSs lookup on HTTP web site
#                        * Setup eS-WIFI WLAN LEDs
#                        * Send a "GET" request to Download the file to your PC
#                          over UART into the Python folder
#                        * Close
#-------------------------------------------------------------------------------
import math, sys, serial, serial.tools.list_ports, time, binascii, urlparse, os

os.linesep ="\n"

#Quiet mode, 0 = Quiet, 1 = Verbose(See all AT commands)
quiet = 0

#Send timeout (sec)
timeout = 20

#Show Hex data, 0 = Don't show, 1 = Show
showhex = 0

#Send a command to a eS-WiFi module and wait for prompt("\r\n> ") responce
def sendCMDBlocking(serPort,cmd):
    rcvString =""

    #Send Command
    if (quiet == 1):
        sys.stdout.write(cmd)
    serPort.write(cmd + '\r')

    st = time.time()                                        #Get time

    #read until "\r\n> " received
    found = 0
    while found != 1:
        btr = serPort.inWaiting()
        rcvString += serPort.read(btr)
        sl = len(rcvString)
        if  '\r\n> ' in rcvString[sl-4:]:
                found = 1
        #Endif

        #Check for timeout
        if (time.time() - st) > timeout:
            return '-1'                                     #Timeout error
        #Endif
    #End while

    #Show responce
    if (quiet == 1):
        if ( '\0' in rcvString[2:]):
            #print ''
            #print "raw:\r\n" + binascii.hexlify(rcvString)
            #print ''
            print rcvString[:1]
            print binascii.hexlify(rcvString[2:(len(rcvString)-8)])
            print rcvString[len(rcvString)-8:]
##        else:
##            sys.stdout.write(rcvString)
        #End if
##    else:
##        print "\r\n"
    #End if

    #Get payload if OK responce
    if ('OK\r\n> ' in rcvString):
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString


#Send data to a eS-WiFi module and wait for prompt("\r\n> ") responce
#   Using a byte array with the S3 command embedded in array
#   e.g. S3=100\r<100 bytes of data>
def sendDataBlocking(serPort,data):
    rcvString =""

    #Send Command
    if (quiet == 1):
        #sys.stdout.write(data)
        if ('S0' in data):
            print(str(data[0:3]))
            if (showhex == 1): print binascii.hexlify(data[4:len(data)])
        #End if
        if ('S3' in data):
            tmp = str(data[:8])
            end = tmp.find('\r')
            print(data[:end])
            if (showhex ==1): print binascii.hexlify(data[end+1:len(data)])
        #end if
    #End if
    serPort.write(data)

    st = time.time()                                        #Get time

    #read until "\r\n> " received
    found = 0
    while found != 1:
        for c in serPort.read():
            rcvString += c
            if  '\r\n> ' in rcvString:
                found = 1
            #Endif
        #End for

        #Check for timeout
        if (time.time() - st) > timeout:
            return '-1'                                     #Timeout error
        #Endif
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString[2:])

    #Get payload if OK responce
    if ('OK\r\n> ' in rcvString):
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString

#Send CMD and wait for responce, waiting nsec seconds before timing out
def sendCMDWait(serPort,cmd,responce,nsec):
    rcvString =""

    #Send Command
    if (quiet == 1):
        sys.stdout.write(cmd)
    serPort.write(cmd + '\r')

    st = time.time()                                        #Get time

    #Check each message recieved for responce
    found = 0
    while found != 1:

        #read until "\r\n> " received
        found0 = 0
        while found0 != 1:
            for c in serPort.read():
                rcvString += c
                #if (quiet == 1):
                #sys.stdout.write(c)
                if  "\r\n> " in rcvString:
                    found0 = 1
                #Endif
            #End for

            #Check for timeout
            if (time.time() - st) > nsec:
                return '-1'                                 #Timeout error
            #Endif
        #End while

        #Show responce
        if (quiet == 1):
            sys.stdout.write(rcvString)

        #Check for responce
        if  responce in rcvString:
            found = 1
        else:
            rcvString = ""                                  #Clear string
        #End if
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString)

    #Get payload if OK responce
    if 'OK\r\n> ' in rcvString:
        rcvString=rcvString[2:(len(rcvString)-8)]
    #Endif

    #Return payload or Error
    return rcvString

#Send wait for responce, waiting nsec seconds before timing out
def waitResponce(serPort,responce,term,nsec):
    rcvString =""
    serPort.timeout = 10

    st = time.time()                                        #Get time

    #Check each message recieved for responce
    found = 0
    while found != 1:

        #read until termination received
        found0 = 0
        while found0 != 1:
            for c in serPort.read():
                rcvString += c
                #if (quiet == 1):
                #    sys.stdout.write(c)
                if  term in rcvString:
                    found0 = 1
                #Endif
            #End for

            #Check for timeout
            if (time.time() - st) > nsec:
                return '-1'                                 #Timeout error
            #Endif
        #End while

        #Check for responce
        if  responce in rcvString:
            found = 1
        else:
            rcvString = ""                                  #Clear string
        #End if
    #End while

    #Show responce
    if (quiet == 1):
        sys.stdout.write(rcvString)

    #Return payload or Error
    return rcvString


#Search COM port for and eS-WiFi module
def findModule(timeout):
    rcvString =""

    #Get ports
    portList = list(serial.tools.list_ports.comports())
    ##print(portList)

    #Search for module
    for index, openPort in enumerate(portList):

        #Serial Port Setup
        port = portList[index][0]
        ##print port
        baud = 115200
        serPort = serial.Serial(port,baud)
        serPort.timeout = 10

        serPort.write('\r')                                 #Send CR

        st = time.time()                                    #Get time

        #read until "\r\n> " received
        found = 0
        while found != 1:
            for c in serPort.read():
                rcvString += c
                if  '\r\n> ' in rcvString:
                    found = 1
                #Endif
            #End for

            #Check for timeout
            if (time.time() - st) > timeout:
                return '-1'                                 #Timeout error
            #Endif
        #End while

        #Try to get Firmware information
        rcv = sendCMDBlocking(serPort,"I?")

        #Close serial port
        serPort.close()

        #Found?
        if (found == 1):
            #print("Port = " + port)
            if "ISM4" in rcv:
                #print(rcv)
                return port
            #Endif
        #End if
    #End for
    return -1

def readInput(message, current, needEval):

    i = raw_input("{0}({1}): ".format(message, current))
    i = i.strip('\r\n ')

    if i == "":
        return current

    if (needEval):
        return eval(i)

    return i


#Application
def main():

    #Redirect std err to std out
    sys.stderr = sys.stdout

    #Start Application
    print( "**** eS-WiFi TCP/UDP File Download Demo ****" )

    #Serial Port Setup
    port = findModule(10)
    if (port == -1):
        print( "Failed to find an eS-WiFi module!")
        sys.exit(-1)
    #End if
    baud = 115200
    ser = serial.Serial(port,baud)
    ser.timeout = 10


    #Setup ---------------------------------------------------------------------
    bd = 115200                     #Default Buad Rate

    #Baud Rate Setup
    #Choose baud rate for demo (uncomment choice)
    chgBaudRate = 0                 #Enable change baud rate, 0=Disable, 1=Enable
    #bd = 115200
    #bd = 460800
    #bd = 921600
    #bd = 1152000
    #bd = 1382400
    bd = 1843200
    #bd = 2073600
    #bd = 3916800

    #Socket/Protocol Setup
    SOCKET = "0"                    #Select Socket 0-3
    PROTOCOL="0"                    #TCP=0, UDP=1, UDP-Lite=2 TCP-SSL(TLS)=3(Hercules doesn't support)

    #TCP Keep-Alive
    TTI = 3000                      #Time-To-Idle (ms)

    #Local Network Setup
    SSID = "Your_SSID"              #SSID of AP
    PSWD = "Your_PSWD"              #Password of AP
    SEC="4"                         #Security type of AP: 0-Open, 1-WEP, 2-WPA, 3-WPA2-AES, 4-WPA2-Mixed
    DHCP="1"                        #Get IP address for DHCP, 0-No, 1-Yes

    #Client Mode
    remoteIP = "192.168.10.100"     #Remote Server IP address (use 192.168.10.100 with AD Direct Mode)
    remotePort = "8002"             #Remote TCP port

    #Host
    host_max_nob = 1460             #The maximum number of bytes(nob) to send to host, 1460 max for IP packets

    #File
    save_to_file = 1                #0-No, 1-Yes
   #---------------------------------------------------------------------------

    if chgBaudRate == 1:
        rcv = sendCMDBlocking(ser, "U2=" + str(bd))         #Set Baud
        ser.write("U0\r")                                   #Start
        time.sleep(.1)                                      #Wait 100ms
        ser.close()                                         #Close
        ser = serial.Serial(port,bd)                        #Reopen
        rcv = sendCMDBlocking(ser, "U?")                    #UART Info
        print ""
        print( "Communications: " + ser.portstr + "@" + str(bd))
    else:
        print ""
        print( "Communications: " + ser.portstr + "@" + str(baud))
    #End if


    #Set default URL
    # Format: http(s)://<domaim:port/path/file name
    # Please note that URL is required to specify port even if the expect port
    # is 80(http) or 443(https)
    URL     = "Format: http(s)://<domaim:port/path/file name"

    #Get input form users
    SSID    = str(readInput("SSID", SSID, False))
    PSWD    = str(readInput("Password", PSWD, False))
    SEC     = str(readInput("Security", SEC, False))
    URL     = str(readInput("URL", URL, False))

    # Read parms
    R2           = "50"         #ms
    RETRY_COUNT  = 40           #max retry time = RS * RETRY_COUNT * max retry loop time
    SHOW_RETREIS = 1            #0=No show, 1=Show
    READCMD      = "R0"         #R0 or R4, When using R4 the four bytes size will stripped
                                #from the response
    if (READCMD != "R0"):
        print("Using " + READCMD + " command")


    #Additional features
    rcv = sendCMDBlocking(ser, "WL=4,3,1")                   #WLAN LEDs (Green Link, Red Activity), Active High
    rcv = sendCMDBlocking(ser, "MT=1")                       #No Usage message on Error
    ##rcv = sendCMDBlocking(ser, "$$$")                       #Verbose

    #Join Network
    rcv = sendCMDBlocking(ser, "CS")                        #Is connected?
    if (rcv == '1'):
        rcv = sendCMDBlocking(ser, "P6=0")                  #Dis-connect
        rcv = sendCMDBlocking(ser, "CD")                    #Dis-connect
    #End if

    rcv = sendCMDBlocking(ser, "C1="+SSID)                  #Set SSID
    rcv = sendCMDBlocking(ser, "C2="+PSWD)                  #Set Password
    rcv = sendCMDBlocking(ser, "C3="+SEC)                   #Security Type: 0-Open, 1-WEP, 2-WPA, 3-WPA2-AES, 4-WPA2-Mixed
    rcv = sendCMDBlocking(ser, "C4="+DHCP)                  #Use DHCP

    #Get start time
    st = time.time()

    #Join
    rcv = sendCMDBlocking(ser, "C0")                        #Join network
    if (rcv != '-1'):
        if "ERROR" in rcv:                                  #Check for error
            print( "Join failed to connect to network\r\n> ")
            sys.exit(-1)
        else:
            print( "Time: " + ("%012.3F" % (time.time()-st)) + " Joining: " + rcv[10:] )
        #End if
    else:
        print( "Joined timed out" )
        sys.exit(-1)
    #End if

    #Setup and Start Client TCP Keep-Alive
    if ((PROTOCOL == "0") or (PROTOCOL == "3")):
        rcv = sendCMDBlocking(ser, "PK=1," + str(TTI))          #TCP Keep-Alive(ms)
    #Endif

    #Start
    print ""
    print( "**** STARTING  ****" )


    #Parse URL
    parsed = urlparse.urlparse(URL)

    htype = URL[:5]
    domain = parsed.hostname
    remotePort = str(parsed.port)
    file_path = parsed.path
    file_name = os.path.basename (file_path)
    print "url:         " + URL
    print "domain:      " + domain
    print "remote port: " + remotePort
    print "file path:   " + file_path
    print "file name:   " + file_name
    print ""
    print ""

    #Create Get Request
    print "***** Request *****"
    get_request = "GET " + file_path + " HTTP/1.1\r\n"
    get_request += "Host: " +domain + ":" + remotePort + "\r\n"
    get_request += "Connection: keep-alive \r\n\r\n"
    get_request_size = str(len(get_request))
    print get_request
    ##print "Size: " + get_request_size

    #Open file to receive data
    if (save_to_file == 1):
        #Download to the same folder that the python script is located
        fname = file_name
        file_handle = open(fname,'wb')
    #End if


    #Setup and Start Client
    remoteIP = sendCMDBlocking(ser, "D0="+domain)           #DNS Lookup (Sets P3 value)

    rcv = sendCMDBlocking(ser, "P0=0")                      #Socket 0
    if (htype == "https"):
        print("TLS enabled")
        rcv = sendCMDBlocking(ser, "P1=3")                  #0=TCP, 3=TCP-SSL
    else:
        rcv = sendCMDBlocking(ser, "P1=0")                  #0=TCP, 3=TCP-SSL
    #endif

    rcv = sendCMDBlocking(ser, "P9=0")                      #0=No verification, if SSL
    rcv = sendCMDBlocking(ser, "P4="+remotePort)            #Remote Port
    print "P?=" + sendCMDBlocking(ser, "P?")                #Display Px info
    rcv = sendCMDBlocking(ser, "P6=1")                      #Start Client
    if (rcv == '-1') or ('ERROR' in rcv):                   #Check for timeout or error
        if "ERROR" in rcv:                                  #Check for error
            print( "Failed to connect to server")
            sys.exit(-1)
        #End if
    #End if

    st = time.time()

    rcv = sendCMDBlocking(ser, "R1=1460")                   #Set Receive size large enough to get complete responce
    rcv = sendCMDBlocking(ser, "R2=5000")                   #Read data timeout, make sure long enough to get response

    #Send Get Request
    print "***** Send GET *****"
    rcv = sendCMDBlocking(ser, "P0=0")                      #Socket 0
    rcv = sendDataBlocking(ser, bytearray("S3=" + get_request_size +"\r" + get_request))    #Send data


    #Get HTTP Responce
    #rcv = sendCMDBlocking(ser, "P0=0")                      #Socket 0
    rcv = sendCMDBlocking(ser, READCMD)                     #Read data
    if ( "200 OK" not in rcv):                              #Did we receive a good responce
        print "***** Response *****"
        print rcv
        print( "Failed to get file")
        #Close file
        if(save_to_file == 1):
            print( "Closing file")
            file_handle.close
        #end if
        print( "Shutting down client" )
        rcv = sendCMDBlocking(ser, "P6=0")                  #Stop Client
        sys.exit(-1)
    #End if

    #Remove NOB
    if (READCMD == "R4"):
        rcv = rcv[4:]
    #endif
    #Determine if data is attached
    ##print "reslen = " + str(len(rcv))
    sdata = rcv.find("\r\n\r\n") + 4
    print "***** Response *****"
    print rcv[0:sdata]
    ##print "data start = " + str(sdata)
    edata = + len(rcv)
    data_size = 0
    if (edata-sdata > 0 ):
        ##print "end data  = " + str(edata)
        ##print "size data = "  + str(edata - sdata)
        if (save_to_file == 1):
            data_size += (edata-sdata)
            ##print "data_size = "  + str(data_size)
            file_handle.write(rcv[sdata:edata])             #Wite data
    #End if

    if ("Content-Length" not in rcv):
        rcv = sendCMDBlocking(ser, READCMD)                 #Retry Read data
    #Remove NOB
    if (READCMD == "R4"):
        rcv = rcv[4:]
    #endif
    responces = rcv.split('\r\n')                           #Split up into lines

    for j in range(0,len(responces)):
        param = responces[j].split(':')                     #Get field
        ##print param[0]
        if (param[0] == 'Content-Length'):                  #Find length
            ##print param[1]
            content = param[1]
            break                                           #Get content-length field
    #End for
    content_length = int(content)                           #File length
    print "***** Downloading " + str(content_length) + " bytes *****"


    #Read timeout must be smaller than Send timeout set at begining of this script
    rcv = sendCMDBlocking(ser, "P0=0")                      #Socket 0
    rcv = sendCMDBlocking(ser, "R1=" + str(host_max_nob))   #Receive Size
    rcv = sendCMDBlocking(ser, "R2=" + R2)                  #Read timeout(max polling rate (ms))
    rcv = sendCMDBlocking(ser, "R?")                        #Display Rx info

    #Receive file
    no_data_count = 0
    running = 1
    rcount = RETRY_COUNT
    st = time.time()
    while (running == 1):
        try:
            #Used to get time in retry state
            if (rcount != RETRY_COUNT):
                et_p = time.time()
            else:
                st_p = time.time()
            #endif

            #Socket 0, Client
            #If switching sockets is required the 'P0' command is required
            #rcv = sendCMDBlocking(ser, "P0=0")              #Socket 0
            bdata = bytearray(sendCMDBlocking(ser, READCMD))   #Read data

            #Remove NOB
            if (READCMD == "R4"):
                bdata = bdata[4:]
            #endif

            if (bdata !=""):
                if("-1\r\nERROR" in bdata):
                    running = 0                             #Download complete
                else:
                    ##print "rcv:\r\n" ,binascii.hexlify(bdata)
                    ##print "rlen = " + str(len(bdata))
                    data_size += len(bdata)-0               #Byte received
                    if (save_to_file == 1):
                        file_handle.write(bdata[:len(bdata)-0]) #wWite data
                    #endif
                    ##print "Bytes = " + str(data_size)
                #endif

                #Retry support
                if (rcount != RETRY_COUNT):
                    if (SHOW_RETREIS == 1):
                        print( "Retries: " + "%04d" % (RETRY_COUNT-rcount) + ", " + "%0.4F" % (et_p-st_p))
                    #endif
                #endif
                rcount = RETRY_COUNT
            else:
                if (rcount == 0):
                    running = 0                             #Download complete
                else:
                    rcount -= 1                             #Decrement retry counter
                #endif
            #endif

            if (data_size == content_length):
                running = 0

        except KeyboardInterrupt:
            #Use Ctrl-C to exit, in PyScripter click Abort debugging icon (Red Square) or press Ctrl-Alt-F9
            running = 0
    #End while
    et = time.time()
    print( "***** Download Complete *****" )
    print""

    print""
    print( "**** Totals ****" )
    print("Total bytes expected  = " + str(content_length))
    print("Total bytes receieved = " + str(data_size))
    print("Total Time(sec)       = " + "%0.4F" % (et-st))
    print("Total kbytes/sec      = " + "%0.1F" % ( (data_size) / (et-st) / 1000 ) )
    print("Total bits/sec        = " + "%0.1F" % ( ((data_size) / (et-st)) * 8 ) )

    #Close file
    if(save_to_file == 1):
        file_handle.close
    #end if

    #Stop Client
    print ""
    print( "**** Complete ****" )
    rcv = sendCMDBlocking(ser, "P0=0")                      #Socket 0
    rcv = sendCMDBlocking(ser, "P6=0")                      #Stop Client


    #Leave Network
    rcv = sendCMDBlocking(ser, "CD")                        #Shutdown SoftAP


    #Reset Baud rate
    if chgBaudRate == 1:
        bd = 115200
        rcv = sendCMDBlocking(ser, "U2=" + str(bd))         #Set Baud
        ser.write("U0\r")                                   #Start
        time.sleep(.1)                                      #Wait 100ms
        ser.close()                                         #Close
        ser = serial.Serial(port,bd)                        #Reopen
        rcv = sendCMDBlocking(ser, "U?")                    #UART Info
        #print(rcv)
    #End if


    #Close serial port
    ser.close()

if __name__ == '__main__':
    sys.exit(main())

